from datetime import timedelta, date, datetime
from math import ceil


def daterange(start: date, end: date) -> list:
    """创建指定范围的日期列表

    :param start:起始日期（包括）
    :param end:结束日期（不包括）
    :return: 日期列表
    """
    return [start + timedelta(n) for n in range(int((end - start).days))]


def from_iso_date(d: str) -> datetime:
    """ ISO-8601 转换成日期

    :param d: ISO-8601 格式字符串
    :return: 日期
    """
    return datetime.fromisoformat(d)


def to_iso_date(d: datetime) -> str:
    """ 日期转换成 ISO-8601

    :param d: 日期
    :return: ISO-8601 格式字符串
    """
    return d.isoformat()


def days_ago(n: int) -> date:
    """从今天开始计算n几天前的日期

    :param n: 几天前
    :return: 日期
    """
    return date.today() - timedelta(n)


def days_from_now(n: int) -> date:
    """从今天开始计算n几天后的日期

    :param n: 几天后
    :return: 日期
    """
    return date.today() + timedelta(n)


def add_days(n: int, d=datetime.today()) -> date:
    """从给定的时间计算日期

    :param n: 天数
    :param d: 要计算的日期
    :return:
    """
    return d + timedelta(n)


def days_diff(start: date, end: date) -> int:
    """计算两个日期之间的天数

    :param start:起始日期
    :param end:结束日期
    :return:天数
    """
    return (end - start).days


def months_diff(start: date, end: date) -> int:
    """计算两个日期之前的月差

    :param start: 起始日期
    :param end: 结束日期
    :return: 月份差
    """
    return ceil((end - start).days / 30)


def is_weekday(d: date = datetime.today()) -> bool:
    """检查日期是否是工作日

    :param d: 日期
    :return: 布尔值
    """
    return d.weekday() <= 4


def is_weekend(d: date = datetime.today()):
    """检查日期是否是周末

    :param d:日期
    :return:布尔值
    """
    return d.weekday() > 4
